# coding: utf-8
import ast

from transpiler.phases.utils import make_lnd
from transpiler.utils import linenodata

DUNDER = {
    ast.Eq: "eq",
    ast.NotEq: "ne",
    ast.Lt: "lt",
    ast.Gt: "gt",
    ast.GtE: "ge",
    ast.LtE: "le",
    ast.In: "contains",
    ast.NotIn: "contains",
}

class DesugarCompare(ast.NodeTransformer):
    def visit_Compare(self, node: ast.Compare):
        res = ast.BoolOp(ast.And(), [], **linenodata(node))
        operands = list(map(self.visit, [node.left, *node.comparators]))
        for left, op, right in zip(operands, node.ops, operands[1:]):
            lnd = make_lnd(left, right)
            if type(op) in (ast.In, ast.NotIn):
                left, right = right, left
            call = ast.Call(
                ast.Attribute(left, f"__{DUNDER[type(op)]}__", **lnd),
                [right],
                [],
                **lnd
            )
            if type(op) == ast.NotIn:
                call = ast.UnaryOp(ast.Not(), call, **lnd)
            res.values.append(call)
        if len(res.values) == 1:
            return res.values[0]
        return res

    # def visit_Compare(self, node: ast.Compare):
    #     res = ast.BoolOp(ast.And(), [], **linenodata(node))
    #     operands = list(map(self.visit, [node.left, *node.comparators]))
    #     for left, op, right in zip(operands, node.ops, operands[1:]):
    #         lnd = make_lnd(left, right)
    #         call = ast.Compare(
    #             left,
    #             [op],
    #             [right],
    #             **lnd
    #         )
    #         if type(op) == ast.NotIn:
    #             call = ast.UnaryOp(ast.Not(), call, **lnd)
    #         res.values.append(call)
    #     if len(res.values) == 1:
    #         return res.values[0]
    #     return res
